import json
from dataclasses import asdict
from typing import TYPE_CHECKING, Literal, Optional, Sequence

from nonebot import logger
from nonebot_plugin_orm import async_scoped_session
from sqlalchemy import select, update

from ..models.song import (
    MaiSong,
    SongDifficulties,
    SongDifficulty,
    SongDifficultyUtage,
    SongNotes,
)
from .orm_models import MaiPlayCount
from .orm_models import MaiSong as MaiSongORMModel
from .orm_models import MaiSongAlias, UserBindInfo

if TYPE_CHECKING:
    from ..updater.songs import MusicAliasResponseItem


class UserBindInfoORM:
    @staticmethod
    async def get_user_bind_info(session: async_scoped_session, user_id: str) -> Optional[UserBindInfo]:
        """获取用户绑定信息"""
        result = await session.execute(select(UserBindInfo).where(UserBindInfo.user_id == user_id))
        return result.scalar_one_or_none()

    @staticmethod
    async def set_user_friend_code(session: async_scoped_session, user_id: str, friend_code: str) -> None:
        """
        设置用户好友码
        """
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if bind_info:
            await session.execute(
                update(UserBindInfo).where(UserBindInfo.user_id == user_id).values(friend_code=friend_code)
            )
        else:
            new_bind_info = UserBindInfo(user_id=user_id, friend_code=friend_code)
            session.add(new_bind_info)
        await session.commit()

    @staticmethod
    async def set_lxns_api_key(session: async_scoped_session, user_id: str, api_key: str) -> None:
        """设置用户的落雪咖啡屋 API 密钥"""
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if bind_info:
            await session.execute(
                update(UserBindInfo).where(UserBindInfo.user_id == user_id).values(lxns_api_key=api_key)
            )
        else:
            new_bind_info = UserBindInfo(user_id=user_id, lxns_api_key=api_key)
            session.add(new_bind_info)
        await session.commit()

    @staticmethod
    async def set_diving_fish_import_token(
        session: async_scoped_session, user_id: str, import_token: str, diving_fish_username: str
    ) -> None:
        """设置用户的水鱼查分器导入密钥"""
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if bind_info:
            await session.execute(
                update(UserBindInfo)
                .where(UserBindInfo.user_id == user_id)
                .values(diving_fish_import_token=import_token, diving_fish_username=diving_fish_username)
            )
        else:
            new_bind_info = UserBindInfo(
                user_id=user_id, diving_fish_import_token=import_token, diving_fish_username=diving_fish_username
            )
            session.add(new_bind_info)
        await session.commit()

    @staticmethod
    async def set_maimaipy_identifier(session: async_scoped_session, user_id: str, maimaipy_identifier: str):
        """
        设置用户的 Maimai.py 鉴权凭证
        """
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if bind_info:
            await session.execute(
                update(UserBindInfo)
                .where(UserBindInfo.user_id == user_id)
                .values(maimaipy_identifier=maimaipy_identifier)
            )
        else:
            new_bind_info = UserBindInfo(user_id=user_id, maimaipy_identifier=maimaipy_identifier)
            session.add(new_bind_info)
        await session.commit()

    @staticmethod
    async def set_default_provider(
        session: async_scoped_session, user_id: str, provider: Literal["lxns", "divingfish"]
    ) -> None:
        """
        设置默认查分器

        :raise ValueError: 如果用户未绑定对应查分器账号则抛出该异常
        """
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if not bind_info:
            raise ValueError("用户未绑定任何查分器账号，无法设置默认查分器")
        elif provider == "lxns" and not bind_info.lxns_api_key:
            raise ValueError("用户未设置落雪咖啡屋 API 密钥，无法设置为默认查分器")
        elif provider == "divingfish" and not bind_info.diving_fish_import_token:
            raise ValueError("用户未设置水鱼查分器导入密钥，无法设置为默认查分器")
        await session.execute(
            update(UserBindInfo).where(UserBindInfo.user_id == user_id).values(default_provider=provider)
        )
        await session.commit()

    @staticmethod
    async def unset_user_bind_info(
        session: async_scoped_session, user_id: str, provider: Optional[Literal["lxns", "divingfish", "maimai"]] = None
    ) -> None:
        """
        解绑用户查分器账号

        :param provider: 指定解绑的查分器，若为 None 则解绑所有查分器
        """
        bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)
        if not bind_info:
            return

        if provider is None:
            await session.delete(bind_info)
        elif provider == "lxns":
            await session.execute(update(UserBindInfo).where(UserBindInfo.user_id == user_id).values(lxns_api_key=None))
        elif provider == "divingfish":
            await session.execute(
                update(UserBindInfo)
                .where(UserBindInfo.user_id == user_id)
                .values(diving_fish_import_token=None, diving_fish_username=None)
            )
        elif provider == "maimai":
            await session.execute(
                update(UserBindInfo).where(UserBindInfo.user_id == user_id).values(maimaipy_identifier=None)
            )

        await session.commit()


class MaiSongORM:
    _cache: dict[int, MaiSong] = {}

    @classmethod
    def get_song_sync(cls, song_id: int) -> Optional[MaiSong]:
        """
        同步获取曲目信息（仅从缓存）
        """
        return cls._cache.get(song_id)

    @classmethod
    def update_cache(cls, song: MaiSong) -> None:
        cls._cache[song.id] = song

    @staticmethod
    async def refresh_cache(session: async_scoped_session) -> None:
        """
        刷新缓存，从数据库加载所有曲目信息
        """
        result = await session.execute(select(MaiSongORMModel))
        rows = result.scalars().all()
        for row in rows:
            song = MaiSongORM._convert(row)
            MaiSongORM.update_cache(song)

    @staticmethod
    def _convert(row: MaiSongORMModel) -> MaiSong:
        """
        反序列化 MaiSongORMModel 为 MaiSong
        """

        def convert_difficulty(diffs_data: dict) -> SongDifficulty:
            diff_note = diffs_data.get("notes", {})
            diffs_data["notes"] = SongNotes(
                total=diff_note.get("total", 0),
                tap=diff_note.get("tap", 0),
                touch=diff_note.get("touch", 0),
                hold=diff_note.get("hold", 0),
                slide=diff_note.get("slide", 0),
                break_=diff_note.get("break", 0),
            )
            return SongDifficulty(**diffs_data)

        # 数据库存储为字符串列，需要反序列化为字典
        diffs = row.difficulties if isinstance(row.difficulties, dict) else json.loads(row.difficulties)
        standard_difficulties = [convert_difficulty(d) for d in diffs.get("standard", [])]
        dx_difficulties = [convert_difficulty(d) for d in diffs.get("dx", [])]
        utage_difficulties = [SongDifficultyUtage(**d) for d in diffs.get("utage", [])] if diffs.get("utage") else None
        return MaiSong(
            id=row.id,
            title=row.title,
            artist=row.artist,
            genre=row.genre,
            bpm=row.bpm,
            map=row.map,
            version=row.version,
            difficulties=SongDifficulties(standard=standard_difficulties, dx=dx_difficulties, utage=utage_difficulties),
        )

    @staticmethod
    async def save_song_info(session: async_scoped_session, song: MaiSong) -> None:
        """
        保存曲目信息到数据库
        """
        difficulties_dict = {
            "standard": [asdict(d) for d in song.difficulties.standard],
            "dx": [asdict(d) for d in song.difficulties.dx],
            "utage": [asdict(d) for d in song.difficulties.utage] if song.difficulties.utage else [],
        }
        song_obj = MaiSongORMModel(
            id=song.id,
            title=song.title,
            artist=song.artist,
            genre=song.genre,
            bpm=song.bpm,
            map=song.map,
            version=song.version,
            # ORM 列为 String，这里序列化为 JSON 字符串
            difficulties=json.dumps(difficulties_dict, ensure_ascii=False),
        )
        session.add(song_obj)
        await session.commit()
        MaiSongORM.update_cache(song)

    @staticmethod
    async def save_song_info_batch(session: async_scoped_session, songs: list[MaiSong]) -> None:
        """
        批量保存曲目信息到数据库，如果存在则更新
        """
        if not songs:
            return

        # 查询已存在的记录
        song_ids = [song.id for song in songs]
        result = await session.execute(select(MaiSongORMModel).where(MaiSongORMModel.id.in_(song_ids)))
        existing_map = {s.id: s for s in result.scalars().all()}

        for song in songs:
            difficulties_dict = {
                "standard": [asdict(d) for d in song.difficulties.standard],
                "dx": [asdict(d) for d in song.difficulties.dx],
                "utage": [asdict(d) for d in song.difficulties.utage] if song.difficulties.utage else [],
            }
            difficulties_json = json.dumps(difficulties_dict, ensure_ascii=False)

            if song.id in existing_map:
                # 更新现有记录
                existing_obj = existing_map[song.id]
                existing_obj.title = song.title
                existing_obj.artist = song.artist
                existing_obj.genre = song.genre
                existing_obj.bpm = song.bpm
                existing_obj.map = song.map
                existing_obj.version = song.version
                existing_obj.difficulties = difficulties_json  # type: ignore
            else:
                # 插入新记录
                song_obj = MaiSongORMModel(
                    id=song.id,
                    title=song.title,
                    artist=song.artist,
                    genre=song.genre,
                    bpm=song.bpm,
                    map=song.map,
                    version=song.version,
                    # ORM 列为 String，这里序列化为 JSON 字符串
                    difficulties=difficulties_json,
                )
                session.add(song_obj)

        await session.commit()
        for song in songs:
            MaiSongORM.update_cache(song)

    @staticmethod
    async def get_song_info(session: async_scoped_session, song_id: int) -> MaiSong:
        """
        获取曲目信息，如果数据库中不存在则从远程获取并保存

        :param song_id: 曲目 ID
        """
        if song_id in MaiSongORM._cache:
            return MaiSongORM._cache[song_id]

        result = await session.execute(select(MaiSongORMModel).where(MaiSongORMModel.id == song_id))
        song_row = result.scalar_one_or_none()
        if song_row:
            return MaiSongORM._convert(song_row)

        logger.warning(f"曲目 ID {song_id} 不存在于数据库，正在从远程获取...")
        from ..updater.songs import fetch_song_info

        song_info = await fetch_song_info(song_id)
        await MaiSongORM.save_song_info(session, song_info)
        return song_info

    @staticmethod
    async def get_songs_info_by_ids(session: async_scoped_session, song_ids: list[int]) -> list[MaiSong]:
        """
        批量获取曲目信息：
        - 先用 IN 查询一次性取回数据库中已有的记录；
        - 对缺失的 ID 再调用远程接口获取并落库；
        - 返回顺序与传入的 song_ids 一致，并去重。
        """
        if not song_ids:
            return []

        # 去重但保持原始顺序
        ordered_unique_ids: list[int] = []
        seen_ids: set[int] = set()
        for sid in song_ids:
            sid_int = int(sid)
            if sid_int not in seen_ids:
                seen_ids.add(sid_int)
                ordered_unique_ids.append(sid_int)

        result = await session.execute(select(MaiSongORMModel).where(MaiSongORMModel.id.in_(ordered_unique_ids)))
        rows = result.scalars().all()

        # 已有记录映射
        id_to_song: dict[int, MaiSong] = {row.id: MaiSongORM._convert(row) for row in rows}

        # 远程补齐缺失记录
        missing_ids = [sid for sid in ordered_unique_ids if sid not in id_to_song]
        for mid in missing_ids:
            logger.warning(f"曲目 ID {mid} 不存在于数据库，正在从远程获取...")
            from ..updater.songs import fetch_song_info

            fetched = await fetch_song_info(mid)
            await MaiSongORM.save_song_info(session, fetched)
            id_to_song[mid] = fetched

        return [id_to_song[sid] for sid in ordered_unique_ids if sid in id_to_song]

    @staticmethod
    async def get_song_info_by_name_or_alias(session: async_scoped_session, name_or_alias: str) -> list[MaiSong]:
        """
        通过乐曲名称或别名获取曲目信息

        :param name_or_alias: 乐曲名称或别名
        :return: 曲目信息 或 None
        """
        # 先尝试通过乐曲名称查找
        result = await session.execute(select(MaiSongORMModel).where(MaiSongORMModel.title.ilike(f"%{name_or_alias}%")))
        song_rows = result.scalars().all()
        songs = []
        if song_rows:
            songs = [MaiSongORM._convert(row) for row in song_rows]
        else:
            # 如果名称查找失败，则通过别名查找
            songs = await MaiSongAliasORM.find_song_by_alias(session, name_or_alias)

        for song in songs:
            # 如果存在精确匹配的结果，直接忽略别的模糊匹配结果
            if song.title == name_or_alias:
                return [song]

        return songs

    @staticmethod
    async def get_all_song_ids(session: async_scoped_session) -> Sequence[int]:
        """
        获取数据库中所有曲目的 ID 列表
        """
        result = await session.execute(select(MaiSongORMModel.id))
        song_ids = result.scalars().all()
        return song_ids


class MaiSongAliasORM:
    @staticmethod
    async def get_aliases(session: async_scoped_session, song_id: int) -> list[str]:
        """
        获取曲目的所有别名

        :param song_id: 曲目 ID
        """
        result = await session.execute(
            select(MaiSongAlias.alias, MaiSongAlias.custom_alias).where(MaiSongAlias.song_id == song_id)
        )
        alias_row = result.one_or_none()
        if not alias_row:
            return []
        aliases = json.loads(alias_row[0]) if alias_row[0] else []
        custom_aliases = json.loads(alias_row[1]) if alias_row[1] else []
        return list(set(aliases + custom_aliases))

    @staticmethod
    async def update_aliases(
        session: async_scoped_session, song_id: int, aliases: list[str], commit: bool = True
    ) -> None:
        """
        更新曲目的别名列表

        :param song_id: 曲目 ID
        :param aliases: 新的别名列表
        :param commit: 是否在更新后提交事务
        """
        await session.execute(
            update(MaiSongAlias)
            .where(MaiSongAlias.song_id == song_id)
            .values(alias=json.dumps(aliases, ensure_ascii=False))
        )

        if commit:
            await session.commit()

    @staticmethod
    async def add_custom_alias(session: async_scoped_session, song_id: int, custom_alias: str) -> None:
        """
        添加自定义别名到曲目

        :param song_id: 曲目 ID
        :param custom_alias: 自定义别名
        """
        result = await session.execute(select(MaiSongAlias).where(MaiSongAlias.song_id == song_id))
        alias_entry = result.scalar_one_or_none()
        if alias_entry:
            current_custom_aliases = json.loads(alias_entry.custom_alias) if alias_entry.custom_alias else []
            if custom_alias not in current_custom_aliases:
                current_custom_aliases.append(custom_alias)
                alias_entry.custom_alias = json.dumps(current_custom_aliases, ensure_ascii=False)
                session.add(alias_entry)
        else:
            new_alias_entry = MaiSongAlias(
                song_id=song_id, alias="[]", custom_alias=json.dumps([custom_alias], ensure_ascii=False)
            )
            session.add(new_alias_entry)
        await session.commit()

    @staticmethod
    async def add_alias_batch(session: async_scoped_session, song_to_aliases: "list[MusicAliasResponseItem]") -> None:
        """
        批量添加曲目别名

        :param song_to_aliases: 曲目 ID 到别名列表的映射
        """
        for item in song_to_aliases:
            song_id = item["song_id"]
            aliases = item["aliases"]
            if not aliases:
                continue

            # 如果已有记录则更新别名列表
            if (
                await session.execute(select(MaiSongAlias).where(MaiSongAlias.song_id == song_id))
            ).scalar_one_or_none():
                await MaiSongAliasORM.update_aliases(session, song_id, aliases, commit=False)
                continue

            alias_entry = MaiSongAlias(song_id=song_id, alias=json.dumps(aliases, ensure_ascii=False))
            session.add(alias_entry)

        await session.commit()

    @staticmethod
    async def find_song_by_alias(session: async_scoped_session, alias: str) -> list[MaiSong]:
        """
        通过别名查找曲目信息

        :param alias: 曲目别名
        :return: 曲目信息 或 None
        """
        # 仅选择需要的列，避免在异步环境中触发 ORM 懒加载（MissingGreenlet）
        result = await session.execute(select(MaiSongAlias.song_id).where(MaiSongAlias.alias.like(f"%{alias}%")))
        song_ids = result.scalars().all()
        if not song_ids:
            return []

        # 批量获取已存在的记录，并保持输入顺序
        return await MaiSongORM.get_songs_info_by_ids(session, [int(sid) for sid in song_ids])


class MaiPlayCountORM:
    @staticmethod
    async def upsert_user_play_counts(
        session: async_scoped_session, user_id: str, records: list[tuple[int, int, int]]
    ) -> int:
        """
        批量写入用户游玩次数。

        :param records: (song_id, difficulty, play_count)
        :return: 实际处理的记录数量
        """
        if not records:
            return 0

        unique_map: dict[tuple[int, int], int] = {}
        for song_id, difficulty, play_count in records:
            unique_map[(song_id, difficulty)] = play_count

        updated = len(unique_map)

        song_ids = [key[0] for key in unique_map]
        result = await session.execute(
            select(MaiPlayCount).where(MaiPlayCount.user_id == user_id, MaiPlayCount.song_id.in_(song_ids))
        )
        existing_rows = result.scalars().all()
        existing_map = {(row.song_id, row.difficulty): row for row in existing_rows}

        for (song_id, difficulty), play_count in unique_map.items():
            row = existing_map.get((song_id, difficulty))
            if row and row.play_count != play_count:
                row.play_count = play_count
            elif not row:
                session.add(
                    MaiPlayCount(
                        user_id=user_id,
                        song_id=song_id,
                        difficulty=difficulty,
                        play_count=play_count,
                    )
                )
            else:
                updated -= 1

        await session.commit()
        return updated

    @staticmethod
    async def get_user_play_count_map(
        session: async_scoped_session, user_id: str, song_ids: list[int]
    ) -> dict[tuple[int, int], int]:
        """
        获取用户游玩次数映射
        """
        if not song_ids:
            return {}

        result = await session.execute(
            select(MaiPlayCount).where(MaiPlayCount.user_id == user_id, MaiPlayCount.song_id.in_(song_ids))
        )
        rows = result.scalars().all()
        return {(row.song_id, row.difficulty): row.play_count for row in rows}

    @staticmethod
    async def get_all_user_play_counts(session: async_scoped_session, user_id: str) -> Sequence[MaiPlayCount]:
        """
        获取用户所有游玩次数记录
        """
        result = await session.execute(select(MaiPlayCount).where(MaiPlayCount.user_id == user_id))
        rows = result.scalars().all()
        return rows



"""Score providers registry and NoneBot dependency helpers.

本模块集中完成各个查分器实现类的单例初始化，并提供可用于
NoneBot 依赖注入的获取函数，便于在对话上下文中直接获得实例。
"""

from __future__ import annotations

from typing import Dict

from nonebot import get_driver

from ._base import BaseScoreProvider
from ._schema import PlayerMaiB50, PlayerMaiInfo, PlayerMaiScore
from .providers.diving_fish import DivingFishScoreProvider
from .providers.lxns import LXNSScoreProvider
from .providers.maimai import MaimaiPyScoreProvider

# --- 单例初始化（每个实现类仅初始化一次） ---
_lxns_provider = LXNSScoreProvider()
_divingfish_provider = DivingFishScoreProvider()
_maimaipy_provider = MaimaiPyScoreProvider()


def get_lxns_provider() -> LXNSScoreProvider:
    """获取 落雪咖啡屋 查分器的单例实例。

    可用于 NoneBot 依赖注入：

        from nonebot.params import Depends
        from .score import get_lxns_provider

        async def handler(provider: LXNSScoreProvider = Depends(get_lxns_provider)):
            ...
    """

    return _lxns_provider


def get_divingfish_provider() -> DivingFishScoreProvider:
    """获取 水鱼 查分器的单例实例。

    可用于 NoneBot 依赖注入：

        from nonebot.params import Depends
        from .score import get_divingfish_provider

        async def handler(provider: DivingFishScoreProvider = Depends(get_divingfish_provider)):
            ...
    """

    return _divingfish_provider


def get_maimaipy_provider() -> MaimaiPyScoreProvider:
    """获取 maimai_py 的单例实例。

    可用于 NoneBot 依赖注入：

        from nonebot.params import Depends
        from .score import get_maimaipy_provider

        async def handler(provider: MaimaiPyScoreProvider = Depends(get_maimaipy_provider)):
            ...
    """
    return _maimaipy_provider


def get_all_score_providers() -> Dict[str, BaseScoreProvider]:
    """获取当前已注册的全部查分器实例。

    返回一个简单的字典映射，键为实现名，值为对应的单例实例。
    便于按需遍历或根据配置选择具体实现。
    """

    return {
        "lxns": _lxns_provider,
        "divingfish": _divingfish_provider,
        "maimaipy": _maimaipy_provider,
    }


async def auto_get_score_provider(user_id: str) -> LXNSScoreProvider | DivingFishScoreProvider:
    from nonebot_plugin_orm import get_scoped_session

    from ..database.crud import UserBindInfoORM

    session = get_scoped_session()
    bind_info = await UserBindInfoORM.get_user_bind_info(session, user_id)

    if bind_info and not bind_info.lxns_api_key and bind_info.diving_fish_import_token:
        return _divingfish_provider

    return _lxns_provider


# --- 生命周期清理：在 NoneBot 关闭时释放网络会话 ---
driver = get_driver()


@driver.on_shutdown
async def _close_score_providers():
    for provider in get_all_score_providers().values():
        await provider.close()


__all__ = [
    "BaseScoreProvider",
    "LXNSScoreProvider",
    "DivingFishScoreProvider",
    "get_lxns_provider",
    "get_all_score_providers",
    "auto_get_score_provider",
    "PlayerMaiInfo",
    "PlayerMaiB50",
    "PlayerMaiScore",
]



from io import BytesIO
from typing import List, Tuple, Union

from PIL import Image, ImageDraw, ImageFont

from ..functions.process import LevelProcessData, PlateProcessData, UnfinishedSong
from ._config import FONT_MAIN

# Mapping for difficulties names
DIFF_NAMES = ["Basic", "Advanced", "Expert", "Master", "Re:Master"]


class ProcessPainter:
    def __init__(self, data: Union[PlateProcessData, LevelProcessData]):
        self.data = data
        self.font_size = 20
        self.font = ImageFont.truetype(str(FONT_MAIN), self.font_size)
        self.padding = 20
        self.line_spacing = 6
        self.bg_color = (255, 255, 255)
        self.text_color = (0, 0, 0)
        self.id_color = (70, 70, 200)  # Blueish for ID

    def draw_plate(self, use_difficult: bool = False) -> BytesIO:
        if not isinstance(self.data, PlateProcessData):
            raise ValueError("Data must be PlateProcessData")

        lines = []
        d = self.data

        # Header
        lines.append((f"{d.ver_name}{d.plan} 剩余进度", True))  # Title

        # Difficult mode (filtering for High Level songs)
        if use_difficult:
            # List only difficult songs
            for item in d.difficult_left:
                line_str = self._format_song(item)
                lines.append((line_str, False))

            lines.append((f"剩余合计 {len(d.difficult_left)} 首", False))

        else:
            # Summary mode or Full List if short
            # Logic: If > 60 total, just show summary counts per difficulty
            if d.total_left > 60:
                # Summary
                lists = [d.basic_left, d.advanced_left, d.expert_left, d.master_left, d.remaster_left]
                for i, l in enumerate(lists):
                    if l:
                        lines.append((f"{DIFF_NAMES[i]} 剩余 {len(l)} 首", False))
                lines.append((f"剩余合计 {d.total_left} 首", False))
            else:
                # Full List by difficulty
                lists = [d.basic_left, d.advanced_left, d.expert_left, d.master_left, d.remaster_left]
                for i, l in enumerate(lists):
                    if not l:
                        continue
                    # Header for diff? No, just list them?
                    # Original code: "DiffName Remaining X" then list?
                    # Original code loops and prints.
                    for item in l:
                        line_str = self._format_song(item)
                        lines.append((line_str, False))
                lines.append((f"剩余合计 {d.total_left} 首", False))

        return self._render(lines)

    def draw_level(self) -> BytesIO:
        if not isinstance(self.data, LevelProcessData):
            raise ValueError("Data must be LevelProcessData")

        lines = []
        d = self.data

        # Header
        lines.append((f"{d.level} {d.plan} 进度", True))

        # Stats
        stats = (
            f"总计: {d.counts['total']}  完成: {d.counts['completed']}  "
            f"剩余: {d.counts['unfinished']}  未玩: {d.counts['not_played']}"
        )
        lines.append((stats, False))
        lines.append(("", False))  # Spacer

        # List Unfinished + Not Played
        # Limit length? Older plugin usually lists them.
        # Check count. If > 50?
        # Sort by ID?
        # d.unfinished is (Song, Score, DiffIdx)
        # d.not_played is (Song, DiffIdx)

        # We want to display them.
        # Just mix them?

        # Let's list unfinished (partial score) first
        if d.unfinished:
            lines.append(("--- 未达标 ---", False))
            for s, sc, idx in d.unfinished:
                if sc is None:
                    continue

                status = f"{sc.achievements:.4f}%"
                if sc.fc:
                    status += f" {sc.fc}"
                if sc.fs:
                    status += f" {sc.fs}"
                line = f"No.{s.id} [{DIFF_NAMES[idx]}] {s.title} {status}"
                lines.append((line, False))

        if d.not_played:
            lines.append(("--- 未游玩 ---", False))
            for s, idx in d.not_played:
                line = f"No.{s.id} [{DIFF_NAMES[idx]}] {s.title}"
                lines.append((line, False))

        if not d.unfinished and not d.not_played:
            lines.append(("恭喜！全部达成！", True))

        return self._render(lines)

    def _format_song(self, item: UnfinishedSong) -> str:
        # Format: No.123 [Master] Title Status
        # Original: No.01 123456 [Master] 13.5 Title Status
        s = item.song
        return f"No.{s.id} [{DIFF_NAMES[item.difficulty]}] {item.level_value} {s.title} {item.status}"

    def _render(self, lines: List[Tuple[str, bool]]) -> BytesIO:
        # Calculate size
        max_width: int = 0
        total_height: int = self.padding * 2

        # Font instances
        font_reg = self.font
        # Make a bold title font?
        font_bold = ImageFont.truetype(str(FONT_MAIN), int(self.font_size * 1.5))

        processed_lines = []

        draw_temp = ImageDraw.Draw(Image.new("RGB", (1, 1)))

        for text, is_title in lines:
            f = font_bold if is_title else font_reg
            bbox = draw_temp.textbbox((0, 0), text, font=f)
            w = int(bbox[2] - bbox[0])
            h = int(bbox[3] - bbox[1])
            # Add some vertical spacing
            h += self.line_spacing
            if is_title:
                h += 10

            max_width = max(max_width, w)
            total_height += h
            processed_lines.append((text, f, h))

        width = int(max_width + self.padding * 2)
        height = int(total_height)

        img = Image.new("RGB", (width, height), self.bg_color)
        draw = ImageDraw.Draw(img)

        y = self.padding
        for text, f, h_inc in processed_lines:
            draw.text((self.padding, y), text, font=f, fill=self.text_color)
            y += h_inc

        bio = BytesIO()
        img.save(bio, format="PNG")
        bio.seek(0)
        return bio



from typing import List, Optional

from PIL import Image

from ..functions.recommend_songs import RecommendSong, RecommendSongs
from ..score import PlayerMaiInfo, PlayerMaiScore
from ._base import ScoreBaseImage
from ._config import COVER_DIR, PIC_DIR
from .utils import change_column_width, coloum_width, find_all_clear_rank


class DrawScores(ScoreBaseImage):
    """
    绘制成绩列表图
    """

    def __init__(self, image: Optional[Image.Image] = None) -> None:
        """
        初始化 DrawScore

        :param image: 基础图片对象
        """
        super().__init__(image)

    def _draw_score_background(self) -> None:
        self._im.alpha_composite(self._get_image(PIC_DIR / "aurora.png", size=(1400, 220), convert_rgba=True))
        self._im.alpha_composite(self._get_image(PIC_DIR / "bg_shines.png", convert_rgba=True), (34, 0))
        self._im.alpha_composite(
            self._get_image(PIC_DIR / "rainbow.png", convert_rgba=True), (319, self._im.size[1] - 643)
        )
        self._im.alpha_composite(
            self._get_image(PIC_DIR / "rainbow_bottom.png", size=(1200, 200), convert_rgba=True),
            (100, self._im.size[1] - 343),
        )
        pattern_bg = self._get_image(PIC_DIR / "pattern.png")
        for h in range((self._im.size[1] // 358) + 1):
            self._im.alpha_composite(pattern_bg, (0, (358 + 7) * h))

    def _get_rise_bg(self, level_index: int) -> Image.Image:
        rise_paths = [
            PIC_DIR / "rise_score_basic.png",
            PIC_DIR / "rise_score_advanced.png",
            PIC_DIR / "rise_score_expert.png",
            PIC_DIR / "rise_score_master.png",
            PIC_DIR / "rise_score_remaster.png",
        ]
        return self._get_image(rise_paths[level_index])

    def draw_scorelist(
        self, player_info: PlayerMaiInfo, scores: List[PlayerMaiScore], title: str, page: int = 1, page_size: int = 50
    ) -> Image.Image:
        """
        绘制分页成绩列表

        :param player_info: 玩家信息
        :param scores: 成绩列表
        :param title: 标题
        :param page: 当前页码
        :param page_size: 每个页码展示的成绩长度
        :return: 绘制后的图片
        """
        self.reset_im()
        self._draw_score_background()

        all_clear_rank = find_all_clear_rank(scores)
        self.draw_profile(player_info, all_clear_rank)

        # Draw title
        self._im.alpha_composite(self.title_lengthen_bg, (475, 200))
        if len(title) < 12:
            self._sy.draw(700, 245, 28, title, self.text_color, "mm")
        else:
            self._sy.draw(700, 245, 22, title, self.text_color, "mm")

        scores = scores[(page - 1) * page_size : page * page_size]
        self.whiledraw(scores, 320)

        self.draw_footer()

        return self._im

    def whilerisepic(self, datas: List[RecommendSong], low_score: int, isdx: bool):
        """
        循环绘制上分推荐数据
        """
        scale = 1.3

        def sv(v: int) -> int:
            return int(round(v * scale))

        y = sv(120)
        for index, data in enumerate(datas):
            x = sv(550) if isdx else sv(50)
            y += sv(150) if index != 0 else 0

            rise_bg = self._get_rise_bg(data.level_index)
            rise_bg = rise_bg.resize((sv(rise_bg.size[0]), sv(rise_bg.size[1])))
            self._im.alpha_composite(rise_bg, (x + sv(30), y))

            # Cover
            cover_path = COVER_DIR / f"{data.song_id}.png"
            if not cover_path.exists():
                cover_path = COVER_DIR / f"{data.song_id + 10000}.png"

            if not cover_path.exists():
                # Fallback or handle missing cover
                cover = Image.new("RGBA", (sv(80), sv(80)), (0, 0, 0, 0))
            else:
                cover = Image.open(cover_path).resize((sv(80), sv(80)))

            self._im.alpha_composite(cover, (x + sv(55), y + sv(40)))

            # Song info
            title = data.title
            if coloum_width(title) > 26:
                title = change_column_width(title, 25) + "..."
            self._sy.draw(x + sv(142), y + sv(44), sv(17), title, self.t_color[data.level_index], "lm")
            self._tb.draw(x + sv(55), y + sv(130), sv(12), f"ID: {data.song_id}", self.id_color[data.level_index], "lm")
            self._im.alpha_composite(
                Image.open(PIC_DIR / f'{"DX" if data.type == "dx" else "SD"}.png').resize((sv(30), sv(11))),
                (x + sv(105), y + sv(125)),
            )

            # Score
            self._tb.draw(
                x + sv(210),
                y + sv(81),
                sv(27),
                f"{data.old_achievements:.4f}%",
                self.t_color[data.level_index],
                anchor="mm",
            )
            self._tb.draw(
                x + sv(375),
                y + sv(81),
                sv(27),
                f"{data.target_achievements:.4f}%",
                self.t_color[data.level_index],
                anchor="mm",
            )

            increase_ra = data.target_dx_rating - low_score

            self._tb.draw(
                x + sv(150),
                y + sv(124),
                sv(18),
                f"Diff {data.difficulty_value:.1f}({data.difficulty_value_fit:.2f})",
                self.id_color[data.level_index],
                "lm",
            )
            self._tb.draw(
                x + sv(360),
                y + sv(124),
                sv(18),
                f"Ra {data.target_dx_rating}(+{increase_ra})",
                self.id_color[data.level_index],
                "lm",
            )

    def draw_rise(self, songs: RecommendSongs, min_dx_rating: int) -> Image.Image:
        """
        绘制上分数据表
        """
        title_bg = self.title_lengthen_bg.copy().resize((450, 100))
        self._im.alpha_composite(title_bg, (150, 30))
        self._sy.draw(375, 75, 30, "旧版本谱面推荐", self.text_color, "mm")
        self.whilerisepic(songs.old_version, min_dx_rating, False)
        self._im.alpha_composite(title_bg, (825, 30))
        self._sy.draw(1050, 75, 30, "新版本谱面推荐", self.text_color, "mm")
        self.whilerisepic(songs.new_version, min_dx_rating, True)

        self.draw_footer()
        return self._im
